@@ -22,6 +22,7 @@ import hashlib
 import yaml
 import json
 import xml.etree.ElementTree as XML
+import xml
 from xml.dom import minidom
 import jenkins
 import re
@@ -31,14 +32,15 @@ import copy
 import itertools
 import fnmatch
 from jenkins_jobs.errors import JenkinsJobsException
+import local_yaml
 
 logger = logging.getLogger(__name__)
 MAGIC_MANAGE_STRING = "<!-- Managed by Jenkins Job Builder -->"
 
 
-# Python <= 2.7.3's minidom toprettyxml produces broken output by adding
-# extraneous whitespace around data. This patches the broken implementation
-# with one taken from Python > 2.7.3.
+# Python 2.6's minidom toprettyxml produces broken output by adding extraneous
+# whitespace around data. This patches the broken implementation with one taken
+# from 2.7
 def writexml(self, writer, indent="", addindent="", newl=""):
     # indent = current indentation
     # addindent = indentation to add to higher levels
@@ -67,7 +69,9 @@ def writexml(self, writer, indent="", addindent="", newl=""):
     else:
         writer.write("/>%s" % (newl))
 
-if sys.version_info[:3] <= (2, 7, 3):
+# PyXML xml.__name__ is _xmlplus. Check that if we don't have the default
+# system version of the minidom, then patch the writexml method
+if sys.version_info[:3] < (2, 7, 0) or xml.__name__ != 'xml':
     minidom.Element.writexml = writexml
 
 
@@ -78,7 +82,7 @@ def deep_format(obj, paramdict):
     # limitations on the values in paramdict - the post-format result must
     # still be valid YAML (so substituting-in a string containing quotes, for
     # example, is problematic).
-    if isinstance(obj, str):
+    if isinstance(obj, basestring):
         try:
             result = re.match('^{obj:(?P<key>\w+)}$', obj)
             if result is not None:
@@ -122,9 +126,15 @@ class YamlParser(object):
         self.jobs = []
         self.config = config
         self.registry = ModuleRegistry(self.config)
+        self.path = ["."]
+        if self.config:
+            if config.has_section('job_builder') and \
+                    config.has_option('job_builder', 'include_path'):
+                self.path = config.get('job_builder',
+                                       'include_path').split(':')
 
     def parse_fp(self, fp):
-        data = yaml.load(fp)
+        data = local_yaml.load(fp, search_path=self.path)
         if data:
             if not isinstance(data, list):
                 raise JenkinsJobsException(
@@ -260,6 +270,7 @@ class YamlParser(object):
         checksums = set([])
         for values in itertools.product(*dimensions):
             params = copy.deepcopy(project)
+            params = self.applyDefaults(params)
 
             expanded_values = {}
             for (k, v) in values:
@@ -303,11 +314,14 @@ class YamlParser(object):
 
     def getXMLForJob(self, data):
         kind = data.get('project-type', 'freestyle')
+
+        keep_desc = False
         if self.config:
-            keep_desc = self.config.getboolean('job_builder',
-                                               'keep_descriptions')
-        else:
-            keep_desc = False
+            if self.config.has_section('job_builder') and \
+                    self.config.has_option('job_builder', 'keep_descriptions'):
+                keep_desc = self.config.getboolean('job_builder',
+                                                   'keep_descriptions')
+
         if keep_desc:
             description = data.get("description", None)
         else:
@@ -407,23 +421,25 @@ class ModuleRegistry(object):
             component_data = {}
 
         # Look for a component function defined in an entry point
-        cache_key = '%s:%s' % (component_list_type, name)
-        eps = ModuleRegistry.entry_points_cache.get(cache_key)
+        eps = ModuleRegistry.entry_points_cache.get(component_list_type)
         if eps is None:
-            eps = list(pkg_resources.iter_entry_points(
-                       group='jenkins_jobs.{0}'.format(component_list_type),
-                       name=name))
-            if len(eps) > 1:
-                raise JenkinsJobsException(
-                    "Duplicate entry point found for component type: '{0}',"
-                    "name: '{1}'".format(component_type, name))
-            elif len(eps) == 1:
-                ModuleRegistry.entry_points_cache[cache_key] = eps
-                logger.debug("Cached entry point %s = %s", cache_key,
-                             ModuleRegistry.entry_points_cache[cache_key])
+            module_eps = list(pkg_resources.iter_entry_points(
+                group='jenkins_jobs.{0}'.format(component_list_type)))
+            eps = {}
+            for module_ep in module_eps:
+                if module_ep.name in eps:
+                    raise JenkinsJobsException(
+                        "Duplicate entry point found for component type: "
+                        "'{0}', '{0}',"
+                        "name: '{1}'".format(component_type, name))
+                eps[module_ep.name] = module_ep
 
-        if len(eps) == 1:
-            func = eps[0].load()
+            ModuleRegistry.entry_points_cache[component_list_type] = eps
+            logger.debug("Cached entry point group %s = %s",
+                         component_list_type, eps)
+
+        if name in eps:
+            func = eps[name].load()
             func(parser, xml_parent, component_data)
         else:
             # Otherwise, see if it's defined as a macro
@@ -450,7 +466,7 @@ class XmlJob(object):
         return hashlib.md5(self.output()).hexdigest()
 
     def output(self):
-        out = minidom.parseString(XML.tostring(self.xml))
+        out = minidom.parseString(XML.tostring(self.xml, encoding='UTF-8'))
         return out.toprettyxml(indent='  ', encoding='utf-8')
 
 
@@ -546,20 +562,37 @@ class Builder(object):
     def load_files(self, fn):
         self.parser = YamlParser(self.global_config)
 
-        if hasattr(fn, 'read'):
-            self.parser.parse_fp(fn)
-            return
+        # handle deprecated behavior
+        if not hasattr(fn, '__iter__'):
+            logger.warning(
+                'Passing single elements for the `fn` argument in '
+                'Builder.load_files is deprecated. Please update your code '
+                'to use a list as support for automatic conversion will be '
+                'removed in a future version.')
+            fn = [fn]
 
-        if os.path.isdir(fn):
-            files_to_process = [os.path.join(fn, f)
-                                for f in os.listdir(fn)
-                                if (f.endswith('.yml') or f.endswith('.yaml'))]
-        else:
-            files_to_process = [fn]
+        files_to_process = []
+        for path in fn:
+            if os.path.isdir(path):
+                files_to_process.extend([os.path.join(path, f)
+                                         for f in os.listdir(path)
+                                         if (f.endswith('.yml')
+                                             or f.endswith('.yaml'))])
+            else:
+                files_to_process.append(path)
 
         for in_file in files_to_process:
-            logger.debug("Parsing YAML file {0}".format(in_file))
-            self.parser.parse(in_file)
+            # use of ask-for-permissions instead of ask-for-forgiveness
+            # performs better when low use cases.
+            if hasattr(in_file, 'name'):
+                fname = in_file.name
+            else:
+                fname = in_file
+            logger.debug("Parsing YAML file {0}".format(fname))
+            if hasattr(in_file, 'read'):
+                self.parser.parse_fp(in_file)
+            else:
+                self.parser.parse(in_file)
 
     def delete_old_managed(self, keep):
         jobs = self.jenkins.get_jobs()
