@@ -37,7 +37,7 @@ import jenkins_jobs.modules.base
 from jenkins_jobs.errors import JenkinsJobsException
 
 
-def git(self, xml_parent, data):
+def git(parser, xml_parent, data):
     """yaml: git
     Specifies the git SCM repository for this job.
     Requires the Jenkins `Git Plugin.
@@ -45,8 +45,9 @@ def git(self, xml_parent, data):
 
     :arg str url: URL of the git repository
     :arg str credentials-id: ID of credentials to use to connect (optional)
-    :arg str refspec: refspec to fetch
-    :arg str name: name to fetch
+    :arg str refspec: refspec to fetch (default '+refs/heads/\*:refs/remotes/\
+remoteName/\*')
+    :arg str name: name to fetch (default 'origin')
     :arg list(str) remotes: list of remotes to set up (optional, only needed if
       multiple remotes need to be set up)
 
@@ -54,12 +55,12 @@ def git(self, xml_parent, data):
                  * **refspec** (`string`) - refspec to fetch (optional)
                  * **credentials-id** - ID of credentials to use to connect
                      (optional)
-    :arg list(str) branches: list of branch specifiers to build
+    :arg list(str) branches: list of branch specifiers to build (default '**')
     :arg list(str) excluded-users: list of users to ignore revisions from
-      when polling for changes. (if polling is enabled)
-    :arg list(str) included-regions: list of file/folders to include
-    :arg list(str) excluded-regions: list of file/folders to exclude
-    :arg str local-branch: Checkout/merge to local branch
+      when polling for changes. (if polling is enabled, optional)
+    :arg list(str) included-regions: list of file/folders to include (optional)
+    :arg list(str) excluded-regions: list of file/folders to exclude (optional)
+    :arg str local-branch: Checkout/merge to local branch (optional)
     :arg dict merge:
         :merge:
             * **remote** (`string`) - name of repo that contains branch to
@@ -67,26 +68,35 @@ def git(self, xml_parent, data):
             * **branch** (`string`) - name of the branch to merge to
     :arg str basedir: location relative to the workspace root to clone to
              (default: workspace)
-    :arg bool skip-tag: Skip tagging
-    :arg bool shallow-clone: Perform shallow clone
-    :arg bool prune: Prune remote branches
-    :arg bool clean: Clean after checkout
-    :arg bool fastpoll: Use fast remote polling
-    :arg bool disable-submodules: Disable submodules
-    :arg bool recursive-submodules: Recursively update submodules
+    :arg bool skip-tag: Skip tagging (default false)
+    :arg bool shallow-clone: Perform shallow clone (default false)
+    :arg bool prune: Prune remote branches (default false)
+    :arg bool clean: Clean after checkout (default false)
+    :arg bool fastpoll: Use fast remote polling (default false)
+    :arg bool disable-submodules: Disable submodules (default false)
+    :arg bool recursive-submodules: Recursively update submodules (default
+      false)
     :arg bool use-author: Use author rather than committer in Jenkin's build
-      changeset
-    :arg str git-tool: The name of the Git installation to use
+      changeset (default false)
+    :arg str git-tool: The name of the Git installation to use (default
+      'Default')
     :arg str reference-repo: Path of the reference repo to use during clone
-    :arg str scm-name: The unique scm name for this Git SCM
-    :arg bool wipe-workspace: Wipe out workspace before build
+      (optional)
+    :arg str scm-name: The unique scm name for this Git SCM (optional)
+    :arg bool wipe-workspace: Wipe out workspace before build (default true)
+    :arg bool ignore-notify: Ignore notifyCommit URL accesses (default false)
     :arg str browser: what repository browser to use (default '(Auto)')
-    :arg str browser-url: url for the repository browser
-    :arg str browser-version: version of the repository browser (GitLab)
+    :arg str browser-url: url for the repository browser (required if browser
+      is not '(Auto)', no default)
+    :arg str browser-version: version of the repository browser (GitLab only,
+      default '0.0')
     :arg str project-name: project name in Gitblit and ViewGit repobrowser
+      (optional)
     :arg str choosing-strategy: Jenkins class for selecting what to build
-    :arg str git-config-name: Configure name for Git clone
-    :arg str git-config-email: Configure email for Git clone
+      (default 'default')
+    :arg str git-config-name: Configure name for Git clone (optional)
+    :arg str git-config-email: Configure email for Git clone (optional)
+    :arg str timeout: Timeout for git commands in minutes (optional)
 
     :browser values:
         :auto:
@@ -133,6 +143,7 @@ def git(self, xml_parent, data):
         ('skip-tag', 'skipTag', False),
         ('scm-name', 'scmName', ''),
         ("shallow-clone", "useShallowClone", False),
+        ("ignore-notify", "ignoreNotifyCommit", False),
     ]
 
     choosing_strategies = {
@@ -212,6 +223,13 @@ def git(self, xml_parent, data):
     if 'local-branch' in data:
         XML.SubElement(scm, 'localBranch').text = data['local-branch']
 
+    if 'timeout' in data:
+        ext = XML.SubElement(scm, 'extensions')
+        co = XML.SubElement(ext,
+                            'hudson.plugins.git.extensions.impl.'
+                            'CheckoutOption')
+        XML.SubElement(co, 'timeout').text = str(data['timeout'])
+
     browser = data.get('browser', 'auto')
     browserdict = {'auto': 'auto',
                    'bitbucketweb': 'BitbucketWeb',
@@ -243,7 +261,7 @@ def git(self, xml_parent, data):
                 data.get('browser-version', '0.0'))
 
 
-def repo(self, xml_parent, data):
+def repo(parser, xml_parent, data):
     """yaml: repo
     Specifies the repo SCM repository for this job.
     Requires the Jenkins `Repo Plugin.
@@ -309,7 +327,70 @@ def repo(self, xml_parent, data):
             xe.text = str(val)
 
 
-def svn(self, xml_parent, data):
+def store(parser, xml_parent, data):
+    """yaml: store
+    Specifies the Visualworks Smalltalk Store repository for this job.
+    Requires the Jenkins `Visualworks Smalltalk Store Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/
+    Visualworks+Smalltalk+Store+Plugin>`_
+
+    :arg str script: name of the Store script to run
+    :arg str repository: name of the Store repository
+    :arg str version-regex: regular expression that specifies which pundle
+             versions should be considered (optional)
+    :arg str minimum-blessing: minimum blessing level to consider (optional)
+    :arg str parcel-builder-file: name of the file to generate as input to
+             a later parcel building step (optional - if not specified, then
+             no parcel builder file will be generated)
+    :arg list pundles:
+
+        :(package or bundle): (`dict`): A package or bundle to check
+
+    Example:
+
+    .. literalinclude:: /../../tests/scm/fixtures/store001.yaml
+    """
+    namespace = 'org.jenkinsci.plugins.visualworks_store'
+    scm = XML.SubElement(xml_parent, 'scm',
+                         {'class': '{0}.StoreSCM'.format(namespace)})
+    if 'script' in data:
+        XML.SubElement(scm, 'scriptName').text = data['script']
+    else:
+        raise JenkinsJobsException("Must specify a script name")
+    if 'repository' in data:
+        XML.SubElement(scm, 'repositoryName').text = data['repository']
+    else:
+        raise JenkinsJobsException("Must specify a repository name")
+    pundle_specs = data.get('pundles', [])
+    if not pundle_specs:
+        raise JenkinsJobsException("At least one pundle must be specified")
+    valid_pundle_types = ['package', 'bundle']
+    pundles = XML.SubElement(scm, 'pundles')
+    for pundle_spec in pundle_specs:
+        pundle = XML.SubElement(pundles, '{0}.PundleSpec'.format(namespace))
+        pundle_type = pundle_spec.keys()[0]
+        pundle_name = pundle_spec[pundle_type]
+        if pundle_type not in valid_pundle_types:
+            raise JenkinsJobsException(
+                'pundle type must be must be one of: '
+                + ', '.join(valid_pundle_types))
+        else:
+            XML.SubElement(pundle, 'name').text = pundle_name
+            XML.SubElement(pundle, 'pundleType').text = pundle_type.upper()
+    if 'version-regex' in data:
+        XML.SubElement(scm, 'versionRegex').text = data['version-regex']
+    if 'minimum-blessing' in data:
+        XML.SubElement(scm, 'minimumBlessingLevel').text = \
+            data['minimum-blessing']
+    if 'parcel-builder-file' in data:
+        XML.SubElement(scm, 'generateParcelBuilderInputFile').text = 'true'
+        XML.SubElement(scm, 'parcelBuilderInputFilename').text = \
+            data['parcel-builder-file']
+    else:
+        XML.SubElement(scm, 'generateParcelBuilderInputFile').text = 'false'
+
+
+def svn(parser, xml_parent, data):
     """yaml: svn
     Specifies the svn SCM repository for this job.
 
@@ -318,6 +399,19 @@ def svn(self, xml_parent, data):
       (default '.')
     :arg str workspaceupdater: optional argument to specify
       how to update the workspace (default wipeworkspace)
+    :arg list(str) excluded-users: list of users to ignore revisions from
+      when polling for changes (if polling is enabled; parameter is optional)
+    :arg list(str) included-regions: list of file/folders to include
+      (optional)
+    :arg list(str) excluded-regions: list of file/folders to exclude (optional)
+    :arg list(str) excluded-commit-messages: list of commit messages to exclude
+      (optional)
+    :arg str exclusion-revprop-name: revision svn-property to ignore (optional)
+    :arg bool ignore-property-changes-on-directories: ignore svn-property only
+      changes of directories (default false)
+    :arg bool filter-changelog: If set Jenkins will apply the same inclusion
+      and exclusion patterns for displaying changelog entries as it does for
+      polling for changes (default false)
     :arg list repos: list of repositories to checkout (optional)
 
       :Repo: * **url** (`str`) -- URL for the repository
@@ -330,16 +424,13 @@ def svn(self, xml_parent, data):
              :emulateclean:  - delete unversioned/ignored files then update
              :update:        - do an svn update as much as possible
 
-    Example::
+    Multiple repos example:
 
-      scm:
-        - svn:
-           workspaceupdater: update
-           repos:
-             - url: http://svn.example.com/repo
-               basedir: .
-             - url: http://svn.example.com/repo2
-               basedir: repo2
+    .. literalinclude:: /../../tests/scm/fixtures/svn-multiple-repos-001.yaml
+
+    Advanced commit filtering example:
+
+    .. literalinclude:: /../../tests/scm/fixtures/svn-regions-001.yaml
     """
     scm = XML.SubElement(xml_parent, 'scm', {'class':
                          'hudson.scm.SubversionSCM'})
@@ -370,8 +461,35 @@ def svn(self, xml_parent, data):
     XML.SubElement(scm, 'workspaceUpdater', {'class':
                    'hudson.scm.subversion.' + updaterclass})
 
+    mapping = [
+        # option, xml name, default value
+        ("excluded-regions", 'excludedRegions', []),
+        ("included-regions", 'includedRegions', []),
+        ("excluded-users", 'excludedUsers', []),
+        ("exclusion-revprop-name", 'excludedRevprop', ''),
+        ("excluded-commit-messages", 'excludedCommitMessages', []),
+        ("ignore-property-changes-on-directories", 'ignoreDirPropChanges',
+            False),
+        ("filter-changelog", 'filterChangelog', False),
+    ]
 
-def tfs(self, xml_parent, data):
+    for optname, xmlname, defvalue in mapping:
+        if isinstance(defvalue, list):
+            val = '\n'.join(data.get(optname, defvalue))
+        else:
+            val = data.get(optname, defvalue)
+        # Skip adding xml entry if default is empty and no value given
+        if not val and (defvalue in ['', []]):
+            continue
+
+        xe = XML.SubElement(scm, xmlname)
+        if isinstance(defvalue, bool):
+            xe.text = str(val).lower()
+        else:
+            xe.text = str(val)
+
+
+def tfs(parser, xml_parent, data):
     """yaml: tfs
     Specifies the Team Foundation Server repository for this job.
     Requires the Jenkins `Team Foundation Server Plugin.
@@ -495,7 +613,7 @@ def tfs(self, xml_parent, data):
                                                   'Browser'})
 
 
-def workspace(self, xml_parent, data):
+def workspace(parser, xml_parent, data):
     """yaml: workspace
     Specifies the cloned workspace for this job to use as a SCM source.
     Requires the Jenkins `Clone Workspace SCM Plugin.
