@@ -31,7 +31,9 @@ Example::
 
 
 import xml.etree.ElementTree as XML
+from jenkins_jobs.errors import JenkinsJobsException
 import jenkins_jobs.modules.base
+from jenkins_jobs.modules import hudson_model
 import logging
 import re
 
@@ -105,14 +107,14 @@ def build_gerrit_triggers(xml_parent, data):
 
 
 def build_gerrit_skip_votes(xml_parent, data):
-    outcomes = {'successful': 'onSuccessful',
-                'failed': 'onFailed',
-                'unstable': 'onUnstable',
-                'notbuilt': 'onNotBuilt'}
+    outcomes = [('successful', 'onSuccessful'),
+                ('failed', 'onFailed'),
+                ('unstable', 'onUnstable'),
+                ('notbuilt', 'onNotBuilt')]
 
     skip_vote_node = XML.SubElement(xml_parent, 'skipVote')
     skip_vote = data.get('skip-vote', {})
-    for result_kind, tag_name in outcomes.iteritems():
+    for result_kind, tag_name in outcomes:
         if skip_vote.get(result_kind, False):
             XML.SubElement(skip_vote_node, tag_name).text = 'true'
         else:
@@ -160,7 +162,7 @@ def gerrit(parser, xml_parent, data):
                   (optional)
 
                   :Branch: * **branch-compare-type** (`str`) -- ''PLAIN'',
-                             ''ANT'' or ''REG_EXP'' (optional, defaults to
+                             ''ANT'' or ''REG_EXP'' (optional) (default
                              ''PLAIN'')
                            * **branch-pattern** (`str`) -- Branch name pattern
                              to match
@@ -169,11 +171,18 @@ def gerrit(parser, xml_parent, data):
                   (optional)
 
                   :File Path: * **compare-type** (`str`) -- ''PLAIN'', ''ANT''
-                                or ''REG_EXP'' (optional, defaults to
-                                ''PLAIN'')
+                                or ''REG_EXP'' (optional) (default ''PLAIN'')
                               * **pattern** (`str`) -- File path pattern to
                                 match
 
+                * **topics** (`list`) -- List of topics to match
+                  (optional)
+
+                  :File Path: * **compare-type** (`str`) -- ''PLAIN'', ''ANT''
+                                or ''REG_EXP'' (optional) (default ''PLAIN'')
+                              * **pattern** (`str`) -- Topic name pattern to
+                                match
+
     :arg dict skip-vote: map of build outcomes for which Jenkins must skip
         vote. Requires Gerrit Trigger Plugin version >= 2.7.0
 
@@ -273,6 +282,19 @@ def gerrit(parser, xml_parent, data):
                 XML.SubElement(fp_tag, 'compareType').text = \
                     file_path.get('compare-type', 'PLAIN')
                 XML.SubElement(fp_tag, 'pattern').text = file_path['pattern']
+
+        topics = project.get('topics', [])
+        if topics:
+            topics_tag = XML.SubElement(gproj, 'topics')
+            for topic in topics:
+                topic_tag = XML.SubElement(topics_tag,
+                                           'com.sonyericsson.hudson.plugins.'
+                                           'gerrit.trigger.hudsontrigger.data.'
+                                           'Topic')
+                XML.SubElement(topic_tag, 'compareType').text = \
+                    topic.get('compare-type', 'PLAIN')
+                XML.SubElement(topic_tag, 'pattern').text = topic['pattern']
+
     build_gerrit_skip_votes(gtrig, data)
     XML.SubElement(gtrig, 'silentMode').text = str(
         data.get('silent', False)).lower()
@@ -330,6 +352,123 @@ def pollscm(parser, xml_parent, data):
     XML.SubElement(scmtrig, 'spec').text = data
 
 
+def build_pollurl_content_type(xml_parent, entries, prefix,
+                               collection_name, element_name):
+    namespace = 'org.jenkinsci.plugins.urltrigger.content'
+    content_type = XML.SubElement(
+        xml_parent, '{0}.{1}ContentType'.format(namespace, prefix))
+    if entries:
+        collection = XML.SubElement(content_type, collection_name)
+        for entry in entries:
+            content_entry = XML.SubElement(
+                collection, '{0}.{1}ContentEntry'.format(namespace, prefix))
+            XML.SubElement(content_entry, element_name).text = entry
+
+
+def pollurl(parser, xml_parent, data):
+    """yaml: pollurl
+    Trigger when the HTTP response from a URL changes.
+    Requires the Jenkins `URLTrigger Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/URLTrigger+Plugin>`_
+
+    :arg string cron: cron syntax of when to run (default '')
+    :arg string polling-node: Restrict where the polling should run.
+                              (optional)
+    :arg list urls: List of URLs to monitor
+
+      :URL: * **url** (`str`) -- URL to monitor for changes (required)
+            * **proxy** (`bool`) -- Activate the Jenkins proxy (default false)
+            * **timeout** (`int`) -- Connect/read timeout in seconds
+              (default 300)
+            * **username** (`string`) -- User name for basic authentication
+              (optional)
+            * **password** (`string`) -- Password for basic authentication
+              (optional)
+            * **check-status** (`int`) -- Check for a specific HTTP status
+              code (optional)
+            * **check-etag** (`bool`) -- Check the HTTP ETag for changes
+              (default false)
+            * **check-date** (`bool`) -- Check the last modification date of
+              the URL (default false)
+            * **check-content** (`list`) -- List of content type changes to
+              monitor
+
+              :Content Type: * **simple** (`bool`) -- Trigger on any change to
+                               the content of the URL (default false)
+                             * **json** (`list`) -- Trigger on any change to
+                               the listed JSON paths
+                             * **text** (`list`) -- Trigger on any change to
+                               the listed regular expressions
+                             * **xml** (`list`) -- Trigger on any change to
+                               the listed XPath expressions
+
+    Example:
+
+    .. literalinclude:: /../../tests/triggers/fixtures/pollurl001.yaml
+    """
+
+    valid_content_types = {
+        'simple': ['Simple', '', '', []],
+        'json': ['JSON', 'jsonPaths', 'jsonPath', None],
+        'text': ['TEXT', 'regExElements', 'regEx', None],
+        'xml': ['XML', 'xPaths', 'xPath', None]
+    }
+    urltrig = XML.SubElement(xml_parent,
+                             'org.jenkinsci.plugins.urltrigger.URLTrigger')
+    node = data.get('polling-node')
+    XML.SubElement(urltrig, 'spec').text = data.get('cron', '')
+    XML.SubElement(urltrig, 'labelRestriction').text = str(bool(node)).lower()
+    if node:
+        XML.SubElement(urltrig, 'triggerLabel').text = node
+    entries = XML.SubElement(urltrig, 'entries')
+    urls = data.get('urls', [])
+    if not urls:
+        raise JenkinsJobsException('At least one url must be provided')
+    for url in urls:
+        entry = XML.SubElement(entries,
+                               'org.jenkinsci.plugins.urltrigger.'
+                               'URLTriggerEntry')
+        XML.SubElement(entry, 'url').text = url['url']
+        XML.SubElement(entry, 'proxyActivated').text = \
+            str(url.get('proxy', False)).lower()
+        if 'username' in url:
+            XML.SubElement(entry, 'username').text = url['username']
+        if 'password' in url:
+            XML.SubElement(entry, 'password').text = url['password']
+        if 'check-status' in url:
+            XML.SubElement(entry, 'checkStatus').text = 'true'
+            XML.SubElement(entry, 'statusCode').text = \
+                str(url.get('check-status'))
+        else:
+            XML.SubElement(entry, 'checkStatus').text = 'false'
+            XML.SubElement(entry, 'statusCode').text = '200'
+        XML.SubElement(entry, 'timeout').text = \
+            str(url.get('timeout', 300))
+        XML.SubElement(entry, 'checkETag').text = \
+            str(url.get('check-etag', False)).lower()
+        XML.SubElement(entry, 'checkLastModificationDate').text = \
+            str(url.get('check-date', False)).lower()
+        check_content = url.get('check-content', [])
+        XML.SubElement(entry, 'inspectingContent').text = \
+            str(bool(check_content)).lower()
+        content_types = XML.SubElement(entry, 'contentTypes')
+        for entry in check_content:
+            type_name = entry.keys()[0]
+            if type_name not in valid_content_types:
+                raise JenkinsJobsException('check-content must be one of : %s'
+                                           % ', '.join(valid_content_types.
+                                                       keys()))
+
+            content_type = valid_content_types.get(type_name)
+            if entry[type_name]:
+                sub_entries = content_type[3]
+                if sub_entries is None:
+                    sub_entries = entry[type_name]
+                build_pollurl_content_type(content_types,
+                                           sub_entries,
+                                           *content_type[0:3])
+
+
 def timed(parser, xml_parent, data):
     """yaml: timed
     Trigger builds at certain times.
@@ -408,6 +547,39 @@ def github_pull_request(parser, xml_parent, data):
         data.get('auto-close-on-fail', False)).lower()
 
 
+def gitlab_merge_request(parser, xml_parent, data):
+    """yaml: gitlab-merge-request
+    Build merge requests in gitlab and report results.
+    Requires the Jenkins `Gitlab MergeRequest Builder Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/
+    Gitlab+Merge+Request+Builder+Plugin>`_
+
+    :arg string cron: cron syntax of when to run (required)
+    :arg string project-path: gitlab-relative path to project (required)
+
+    Example:
+
+    .. literalinclude:: \
+        /../../tests/triggers/fixtures/gitlab-merge-request.yaml
+    """
+    ghprb = XML.SubElement(xml_parent, 'org.jenkinsci.plugins.gitlab.'
+                           'GitlabBuildTrigger')
+    if not data.get('cron', None):
+        raise jenkins_jobs.errors.JenkinsJobsException(
+            'gitlab-merge-request is missing "cron"')
+    if not data.get('project-path', None):
+        raise jenkins_jobs.errors.JenkinsJobsException(
+            'gitlab-merge-request is missing "project-path"')
+
+    # Because of a design limitation in the GitlabBuildTrigger Jenkins plugin
+    # both 'spec' and '__cron' have to be set to the same value to have them
+    # take effect. Also, cron and projectPath are prefixed with underscores
+    # in the plugin, but spec is not.
+    XML.SubElement(ghprb, 'spec').text = data.get('cron')
+    XML.SubElement(ghprb, '__cron').text = data.get('cron')
+    XML.SubElement(ghprb, '__projectPath').text = data.get('project-path')
+
+
 def build_result(parser, xml_parent, data):
     """yaml: build-result
     Configure jobB to monitor jobA build result. A build is scheduled if there
@@ -475,6 +647,52 @@ def build_result(parser, xml_parent, data):
             XML.SubElement(model_checked, 'checked').text = result_dict[result]
 
 
+def reverse(parser, xml_parent, data):
+    """yaml: reverse
+    This trigger can be configured in the UI using the checkbox with the
+    following text: 'Build after other projects are built'.
+
+    Set up a trigger so that when some other projects finish building, a new
+    build is scheduled for this project. This is convenient for running an
+    extensive test after a build is complete, for example.
+
+    This configuration complements the "Build other projects" section in the
+    "Post-build Actions" of an upstream project, but is preferable when you
+    want to configure the downstream project.
+
+    :arg str jobs: List (comma separated) of jobs to watch.
+    :arg str result: Build results to monitor for between the following
+      options: success, unstable and failure. (default 'success').
+
+    Example:
+
+    .. literalinclude:: /../../tests/triggers/fixtures/reverse.yaml
+    """
+    reserveBuildTrigger = XML.SubElement(
+        xml_parent, 'jenkins.triggers.ReverseBuildTrigger')
+
+    supported_thresholds = ['SUCCESS', 'UNSTABLE', 'FAILURE']
+
+    XML.SubElement(reserveBuildTrigger, 'spec').text = ''
+    XML.SubElement(reserveBuildTrigger, 'upstreamProjects').text = \
+        data.get('jobs')
+
+    threshold = XML.SubElement(reserveBuildTrigger, 'threshold')
+    result = data.get('result').upper()
+    if result not in supported_thresholds:
+        raise jenkins_jobs.errors.JenkinsJobsException(
+            "Choice should be one of the following options: %s." %
+            ", ".join(supported_thresholds))
+    XML.SubElement(threshold, 'name').text = \
+        hudson_model.THRESHOLDS[result]['name']
+    XML.SubElement(threshold, 'ordinal').text = \
+        hudson_model.THRESHOLDS[result]['ordinal']
+    XML.SubElement(threshold, 'color').text = \
+        hudson_model.THRESHOLDS[result]['color']
+    XML.SubElement(threshold, 'completeBuild').text = \
+        str(hudson_model.THRESHOLDS[result]['complete']).lower()
+
+
 def script(parser, xml_parent, data):
     """yaml: script
     Triggers the job using shell or batch script.
