@@ -24,6 +24,7 @@ Wrappers can alter the way the build is run as well as the build output.
 
 import xml.etree.ElementTree as XML
 import jenkins_jobs.modules.base
+from jenkins_jobs.errors import JenkinsJobsException
 from jenkins_jobs.modules.builders import create_builders
 
 
@@ -56,6 +57,41 @@ def ci_skip(parser, xml_parent, data):
     })
 
 
+def logfilesize(parser, xml_parent, data):
+    """yaml: logfilesize
+    Abort the build if its logfile becomes too big.
+    Requires the Jenkins `Logfilesizechecker Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Logfilesizechecker+Plugin>`_
+
+    :arg bool set-own: Use job specific maximum log size instead of global
+        config value (default false).
+    :arg bool fail: Make builds aborted by this wrapper be marked as "failed"
+        (default false).
+    :arg int size: Abort the build if logfile size is bigger than this
+        value (in MiB, default 128). Only applies if set-own is true.
+
+    Minimum config example:
+
+    .. literalinclude:: /../../tests/wrappers/fixtures/logfilesize002.yaml
+
+    Full config example:
+
+    .. literalinclude:: /../../tests/wrappers/fixtures/logfilesize001.yaml
+
+    """
+    lfswrapper = XML.SubElement(xml_parent,
+                                'hudson.plugins.logfilesizechecker.'
+                                'LogfilesizecheckerWrapper')
+    lfswrapper.set("plugin", "logfilesizechecker")
+
+    XML.SubElement(lfswrapper, 'setOwn').text = str(
+        data.get('set-own', 'false')).lower()
+    XML.SubElement(lfswrapper, 'maxLogSize').text = str(
+        data.get('size', '128')).lower()
+    XML.SubElement(lfswrapper, 'failBuild').text = str(
+        data.get('fail', 'false')).lower()
+
+
 def timeout(parser, xml_parent, data):
     """yaml: timeout
     Abort the build if it runs too long.
@@ -388,22 +424,21 @@ def locks(parser, xml_parent, data):
 
     :arg: list of locks to use
 
-    Example::
+    Example:
 
-      wrappers:
-        - locks:
-            - FOO
-            - FOO2
+    .. literalinclude::  /../../tests/wrappers/fixtures/locks002.yaml
+       :language: yaml
     """
-    lw = XML.SubElement(xml_parent,
-                        'hudson.plugins.locksandlatches.LockWrapper')
-    locktop = XML.SubElement(lw, 'locks')
     locks = data
-    for lock in locks:
-        lockwrapper = XML.SubElement(locktop,
-                                     'hudson.plugins.locksandlatches.'
-                                     'LockWrapper_-LockWaitConfig')
-        XML.SubElement(lockwrapper, 'name').text = lock
+    if locks:
+        lw = XML.SubElement(xml_parent,
+                            'hudson.plugins.locksandlatches.LockWrapper')
+        locktop = XML.SubElement(lw, 'locks')
+        for lock in locks:
+            lockwrapper = XML.SubElement(locktop,
+                                         'hudson.plugins.locksandlatches.'
+                                         'LockWrapper_-LockWaitConfig')
+            XML.SubElement(lockwrapper, 'name').text = lock
 
 
 def copy_to_slave(parser, xml_parent, data):
@@ -483,6 +518,31 @@ def inject(parser, xml_parent, data):
     XML.SubElement(info, 'loadFilesFromMaster').text = 'false'
 
 
+def inject_ownership_variables(parser, xml_parent, data):
+    """yaml: inject-ownership-variables
+    Inject ownership variables to the build as environment variables.
+    Requires the Jenkins `EnvInject Plugin and Jenkins Ownership plugin
+    <https://wiki.jenkins-ci.org/display/JENKINS/EnvInject+Plugin>
+    <https://wiki.jenkins-ci.org/display/JENKINS/Ownership+Plugin>`_
+
+    :arg bool job-variables: inject job ownership variables to the job
+        (default false)
+    :arg bool node-variables: inject node ownership variables to the job
+        (default false)
+
+    Example:
+
+    .. literalinclude:: /../../tests/wrappers/fixtures/ownership001.yaml
+
+    """
+    ownership = XML.SubElement(xml_parent, 'com.synopsys.arc.jenkins.plugins.'
+                               'ownership.wrappers.OwnershipBuildWrapper')
+    XML.SubElement(ownership, 'injectNodeOwnership').text = \
+        str(data.get('node-variables', False)).lower()
+    XML.SubElement(ownership, 'injectJobOwnership').text = \
+        str(data.get('job-variables', False)).lower()
+
+
 def inject_passwords(parser, xml_parent, data):
     """yaml: inject-passwords
     Inject passwords to the build as environment variables.
@@ -536,6 +596,27 @@ def env_file(parser, xml_parent, data):
         eib, 'filePath', data.get('properties-file'))
 
 
+def env_script(parser, xml_parent, data):
+    """yaml: env-script
+    Add or override environment variables to the whole build process.
+    Requires the Jenkins `Environment Script Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/Environment+Script+Plugin>`_
+
+    :arg script-content: The script to run (default: '')
+    :arg only-run-on-parent: Only applicable for Matrix Jobs. If true, run only
+      on the matrix parent job (default: false)
+
+    Example:
+
+    .. literalinclude:: /../../tests/wrappers/fixtures/env-script001.yaml
+
+    """
+    el = XML.SubElement(xml_parent, 'com.lookout.jenkins.EnvironmentScript')
+    XML.SubElement(el, 'script').text = data.get('script-content', '')
+    only_on_parent = str(data.get('only-run-on-parent', False)).lower()
+    XML.SubElement(el, 'onlyRunOnParent').text = only_on_parent
+
+
 def jclouds(parser, xml_parent, data):
     """yaml: jclouds
     Uses JClouds to provide slave launching on most of the currently
@@ -611,7 +692,7 @@ def release(parser, xml_parent, data):
     Requires the Jenkins `Release Plugin.
     <https://wiki.jenkins-ci.org/display/JENKINS/Release+Plugin>`_
 
-    :arg bool keep-forever: Keep build forever (default is 'true')
+    :arg bool keep-forever: Keep build forever (default true)
     :arg bool override-build-parameters: Enable build-parameter override
     :arg string version-template: Release version template
     :arg list parameters: Release parameters (see the :ref:`Parameters` module)
@@ -979,6 +1060,32 @@ def exclusion(parser, xml_parent, data):
         XML.SubElement(dit, 'name').text = str(resource).upper()
 
 
+def ssh_agent_credentials(parser, xml_parent, data):
+    """yaml: ssh-agent-credentials
+    Sets up the user for the ssh agent plugin for jenkins.
+
+    Requires the Jenkins `SSH-Agent Plugin.
+    <https://wiki.jenkins-ci.org/display/JENKINS/SSH-Agent-Plugin>`_
+
+    :arg str user: The user id of the jenkins user credentials (required)
+
+    Example:
+
+    .. literalinclude::
+            /../../tests/wrappers/fixtures/ssh-agent-credentials001.yaml
+
+    """
+
+    entry_xml = XML.SubElement(
+        xml_parent,
+        'com.cloudbees.jenkins.plugins.sshagent.SSHAgentBuildWrapper')
+
+    try:
+        XML.SubElement(entry_xml, 'user').text = data['user']
+    except KeyError:
+        raise JenkinsJobsException("Missing 'user' for ssh-agent-credentials")
+
+
 class Wrappers(jenkins_jobs.modules.base.Base):
     sequence = 80
 
