@@ -22,11 +22,12 @@ import logging
 import os
 import re
 import doctest
+import json
 import operator
 import testtools
 import xml.etree.ElementTree as XML
 from ConfigParser import ConfigParser
-import yaml
+import jenkins_jobs.local_yaml as yaml
 from jenkins_jobs.builder import XmlJob, YamlParser, ModuleRegistry
 from jenkins_jobs.modules import (project_flow,
                                   project_matrix,
@@ -34,30 +35,33 @@ from jenkins_jobs.modules import (project_flow,
                                   project_multijob)
 
 
-def get_scenarios(fixtures_path):
+def get_scenarios(fixtures_path, in_ext='yaml', out_ext='xml'):
     """Returns a list of scenarios, each scenario being described
-    by two parameters (yaml and xml filenames).
-        - content of the fixture .xml file (aka expected)
+    by two parameters (yaml and xml filenames by default).
+        - content of the fixture output file (aka expected)
     """
     scenarios = []
     files = os.listdir(fixtures_path)
-    yaml_files = [f for f in files if re.match(r'.*\.yaml$', f)]
+    input_files = [f for f in files if re.match(r'.*\.{0}$'.format(in_ext), f)]
 
-    for yaml_filename in yaml_files:
-        xml_candidate = re.sub(r'\.yaml$', '.xml', yaml_filename)
-        # Make sure the yaml file has a xml counterpart
-        if xml_candidate not in files:
+    for input_filename in input_files:
+        output_candidate = re.sub(r'\.{0}$'.format(in_ext),
+                                  '.{0}'.format(out_ext), input_filename)
+        # Make sure the input file has a output counterpart
+        if output_candidate not in files:
             raise Exception(
-                "No XML file named '%s' to match "
-                "YAML file '%s'" % (xml_candidate, yaml_filename))
-        conf_candidate = re.sub(r'\.yaml$', '.conf', yaml_filename)
+                "No {0} file named '{1}' to match {2} file '{3}'"
+                .format(out_ext.upper(), output_candidate,
+                        in_ext.upper(), input_filename))
+
+        conf_candidate = re.sub(r'\.yaml$', '.conf', input_filename)
         # If present, add the configuration file
         if conf_candidate not in files:
             conf_candidate = None
 
-        scenarios.append((yaml_filename, {
-            'yaml_filename': yaml_filename,
-            'xml_filename': xml_candidate,
+        scenarios.append((input_filename, {
+            'in_filename': input_filename,
+            'out_filename': output_candidate,
             'conf_filename': conf_candidate,
         }))
 
@@ -74,22 +78,24 @@ class BaseTestCase(object):
 
     logging.basicConfig()
 
-    def __read_content(self):
+    def _read_utf8_content(self):
         # Read XML content, assuming it is unicode encoded
-        xml_filepath = os.path.join(self.fixtures_path, self.xml_filename)
+        xml_filepath = os.path.join(self.fixtures_path, self.out_filename)
         xml_content = u"%s" % codecs.open(xml_filepath, 'r', 'utf-8').read()
+        return xml_content
 
-        yaml_filepath = os.path.join(self.fixtures_path, self.yaml_filename)
+    def _read_yaml_content(self):
+        yaml_filepath = os.path.join(self.fixtures_path, self.in_filename)
         with file(yaml_filepath, 'r') as yaml_file:
             yaml_content = yaml.load(yaml_file)
-
-        return (yaml_content, xml_content)
+        return yaml_content
 
     def test_yaml_snippet(self):
-        if not self.xml_filename or not self.yaml_filename:
+        if not self.out_filename or not self.in_filename:
             return
 
-        yaml_content, expected_xml = self.__read_content()
+        expected_xml = self._read_utf8_content()
+        yaml_content = self._read_yaml_content()
         project = None
         if ('project-type' in yaml_content):
             if (yaml_content['project-type'] == "maven"):
@@ -126,13 +132,9 @@ class BaseTestCase(object):
 
 class SingleJobTestCase(BaseTestCase):
     def test_yaml_snippet(self):
-        if not self.xml_filename or not self.yaml_filename:
-            return
+        expected_xml = self._read_utf8_content()
 
-        xml_filepath = os.path.join(self.fixtures_path, self.xml_filename)
-        expected_xml = u"%s" % open(xml_filepath, 'r').read()
-
-        yaml_filepath = os.path.join(self.fixtures_path, self.yaml_filename)
+        yaml_filepath = os.path.join(self.fixtures_path, self.in_filename)
 
         if self.conf_filename:
             config = ConfigParser()
@@ -150,7 +152,8 @@ class SingleJobTestCase(BaseTestCase):
         parser.jobs.sort(key=operator.attrgetter('name'))
 
         # Prettify generated XML
-        pretty_xml = "\n".join(job.output() for job in parser.jobs)
+        pretty_xml = unicode("\n".join(job.output() for job in parser.jobs),
+                             'utf-8')
 
         self.assertThat(
             pretty_xml,
@@ -159,3 +162,21 @@ class SingleJobTestCase(BaseTestCase):
                                               doctest.NORMALIZE_WHITESPACE |
                                               doctest.REPORT_NDIFF)
         )
+
+
+class JsonTestCase(BaseTestCase):
+
+    def test_yaml_snippet(self):
+        expected_json = self._read_utf8_content()
+        yaml_content = self._read_yaml_content()
+
+        pretty_json = json.dumps(yaml_content, indent=4,
+                                 separators=(',', ': '))
+
+        self.assertThat(
+            pretty_json,
+            testtools.matchers.DocTestMatches(expected_json,
+                                              doctest.ELLIPSIS |
+                                              doctest.NORMALIZE_WHITESPACE |
+                                              doctest.REPORT_NDIFF)
+        )
